
TRUE  = 1 # ACK, YES, success
FALSE = 0 # NAK, NO,  failed

import sys, string

global argv_hex, argv_dec
argv_hex = []
argv_dec = []
for xx in sys.argv:
    if len(xx) > 0:
        argv_hex += [int(xx,16)] if all (yy in '+-*/^%' + string.hexdigits for yy in xx) else [xx]
        argv_dec += [int(xx,10)] if all (yy in '+-*/^%' + string.digits    for yy in xx) else [xx]


def no_argument ():
    if len(sys.argv) < 2:
        f = open (sys.argv[0],'r')
        cmd = ''
        for line in f:
            if line.find ('line')<0 and line.find ('sys.argv[')>=0 and line.find (']==')>0:
                print line,
            if line.find ('line')<0 and line.find ('% python')>=0:
                cmd += '\n' + ' '.join(line.split()[0:])
            if line.find ('line')<0 and line.find ('tstmst_func')>=0:
                basic_path = '\\'.join(sys.argv[0].split('\\')[0:-1]) + '\\basic.py'
                for gg in open (basic_path,'r'):
                    if gg.find ('line')<0 and gg.find ('sys.argv[')>=0 and gg.find (']==')>0:
                        print gg,

        print 'ex:',
        print cmd if len(cmd) else '\n% '+sys.argv[0]
        f.close ()
        return TRUE
    else:
        return FALSE


def tstmst_func (tstmst):
    if   sys.argv[1]=='rev'   : print tstmst.sfr.name
    elif sys.argv[1]=='sfr'   : print tstmst.sfr.get_sfr_name (argv_hex[2])
    elif sys.argv[1]=='adc'   : print tstmst.get_adc10 (argv_hex[2])
    elif sys.argv[1]=='wrx'   : print tstmst.sfrwx (argv_hex[2],argv_hex[3:])
    elif sys.argv[1]=='write' : print tstmst.sfrwx (argv_hex[2],[argv_hex[3]])
    elif sys.argv[1]=='read'  : print '0x%02x' % tstmst.sfrrx (argv_hex[2],1)[0]
    elif sys.argv[1]=='loopr' : tstmst.loopr (100,sys.argv[2:])
    elif sys.argv[1]=='loopw' : tstmst.loopw (100,sys.argv[2:])

    elif sys.argv[1]=='dump'  :
        if len(sys.argv)==2:    tstmst.sfr_form (0x80,0x80)
        else:                   tstmst.sfr_form (argv_hex[2],argv_hex[3])
    elif sys.argv[1]=='nvm'   :
        if len(sys.argv)==2:    tstmst.nvm_form (0x900,0x80)
        else:                   tstmst.nvm_form (argv_hex[2],argv_hex[3])

    elif sys.argv[1]=='stop'  : print tstmst.sfrwx (0xBC,[8]) # stop MCU
    elif sys.argv[1]=='reset' : print tstmst.sfrwx (0xF7,[1,1,1]) # reset MCU
    elif sys.argv[1]=='trim'  : print ['0x%02x' % xx for xx in tstmst.get_trim ()]

    elif sys.argv[1]=='prog_hex'  : tstmst.nvm_prog (argv_hex[3],argv_hex[4:],argv_hex[2])
    elif sys.argv[1]=='prog_asc'  : tstmst.nvm_prog (argv_hex[3],map(ord,list(sys.argv[4])),argv_hex[2])
    elif sys.argv[1]=='prog_str'  : tstmst.nvm_prog_block ( \
                                                     argv_hex[3],map(ord,list(sys.argv[4])),len(sys.argv[4]),argv_hex[2])
    elif sys.argv[1]=='upload'    : tstmst.nvm_upload_block (sys.argv[2],argv_hex[3])
    elif sys.argv[1]=='comp'      : tstmst.nvm_comp_block (sys.argv[2:])
    elif sys.argv[1]=='test'      : tstmst.test ()
    else: print "command not recognized,", sys.argv[1]



import os
if os.name=="nt": # if not, the loop won't break by keyboard
    import msvcrt

    def check_break ():
        if os.name=="nt" and msvcrt.kbhit():
            return ord(msvcrt.getch())
        else:
            return FALSE




TRUE  = 1 # ACK, YES, success
FALSE = 0 # NAK, NO,  failed

class sfr11xx (object):
    '''
    sfr11xx class hierarchy
    -----------------------
            sfr1108
           /
    sfr11xx - sfr111x - sfr1110
                     \
                      sfr1112
    '''
    #############################################################################
    ## M51/CAN11XX common part                                                 ##
    WDTREL  = 0x86

    S0CON   = 0x98
    S0BUF   = 0x99

    IEN0    = 0xA8 ## IE in REG52.H
    IP0     = 0xA9
    S0RELL  = 0xAA

    IP      = 0xB8 ## R8051
    IEN1    = 0xB8 ## R80515
    S0RELH  = 0xBA

    IRCON   = 0xC0

    ADCON   = 0xD8
    I2CDAT  = 0xDA
    I2CADR  = 0xDB
    I2CCON  = 0xDC
    I2CSTA  = 0xDD

    SRST    = 0xF7

    TXCTL   = 0xb0
    FFCTL   = 0xb1
    FFIO    = 0xb2
    STA0    = 0xb3
    STA1    = 0xb4
    MSK0    = 0xb5
    MSK1    = 0xb6
    FFSTA   = 0xb7
    RXCTL   = 0xbb
    MISC    = 0xbc
    PRLS    = 0xbd
    PRLTX   = 0xbe
    GPF     = 0xbf

    I2CCMD  = 0xc1
    OFS     = 0xc2
    DEC     = 0xc3
    PRLRXL  = 0xc4
    PRLRXH  = 0xc5
    TRXS    = 0xc6
    REVID   = 0xc7

    OSCCTL  = 0xd4
    GPIOP   = 0xd5
    GPIOSL  = 0xd6
    GPIOSH  = 0xd7

    I2CCTL  = 0xc9
    I2CDEVA = 0xca
    I2CMSK  = 0xcb
    I2CDEV  = 0xcc
    I2CBUF  = 0xcd
    PCL     = 0xce
    NVMIO   = 0xcf

    CCRX    = 0xe6

    def __init__ (me, revid=0):
        me.bufsz = 34 # -byte FIFO
        me.name = ''
        me.revid = revid
        if revid:
            me.name = me.dict_id [revid]

    def check (me, revid):
        for k,v in me.dict_id.iteritems():
            if k is revid:
                return TRUE
        return FALSE

    def get_sfr_name (me, adr):
        for k,v in list(vars(sfr11xx).iteritems()):
            if v is adr:
                return k
        return '' # not found



class sfr1108 (sfr11xx):

    ANACTL  = 0xd1
    AOPTL   = 0xd2
    AOPTH   = 0xd3

    TM      = 0xd9

    dict_id = {0x0a:'CAN1108', \
               0x1a:'CAN1111'}

    def __init__ (me, revid=0):
        sfr11xx.__init__ (me, revid)
        me.inc = 1 # CAN1108 power-on I2CSLV in INC mode
        me.nbyte = 2
        me.dummy = 3
        me.nvmsz = 0x0a00
        me.nvmmsk = 0x0fff # address width
        me.trimsz = 2
        me.trimnum = 8

    def get_sfr_name (me, adr):
        ret = super(sfr1108,me).get_sfr_name(adr)
        if ret == '':
            for k,v in list(vars(sfr1108).iteritems()):
                if v is adr:
                    return k
        return ret



class sfr111x (sfr11xx):

    RWBUF   = 0xd2

    ATM     = 0xd9

    P0MSK   = 0xde
    P0STA   = 0xdf

    SRCCTL  = 0xe3
    PWRCTL  = 0xe4
    PWR_V   = 0xe5

    DACCTL  = 0xf1
    DACEN   = 0xf2
    SAREN   = 0xf3

    DACV0   = 0xf8
    DACV1   = 0xf9
    DACV2   = 0xfa
    DACV3   = 0xfb
    DACV4   = 0xfc
    DACV5   = 0xfd
    DACV6   = 0xfe
    DACV7   = 0xff

    def get_sfr_name (me, adr):
        ret = super(sfr111x,me).get_sfr_name(adr)
        if ret == '':
            for k,v in list(vars(sfr111x).iteritems()):
                if v is adr:
                    return k
        return ret



class sfr1110 (sfr111x):

    CDCTL   = 0xa1
    CDVAL   = 0xa2
    PWR_I   = 0xa3
    PWMP    = 0xa4
    PWMD    = 0xa5
    PROCTL  = 0xa6
    PROSTA  = 0xa7
    
    CVCTL   = 0xab
    DTR     = 0xac
    DTF     = 0xad
    DDCTL   = 0xae
    DDBND   = 0xaf

    NVMCTL  = 0xd1 # CAN1110

    EXGP    = 0xd3 # CAN1110

    DACLSB  = 0xf6


    dict_id = {0x0b:'CAN1110A/B', \
               0x0c:'CAN1110C/D'}

    def __init__ (me, revid=0):
        sfr11xx.__init__ (me, revid)
        me.inc = 0 # CAN1110 power-on I2CSLV in non-INC mode
        me.nbyte = 1
        me.dummy = 3
        me.nvmsz = 0x2000
        me.nvmmsk = 0x1fff # address width
        me.trimsz = 5
        me.trimnum = 6

    def get_sfr_name (me, adr):
        ret = super(sfr1110,me).get_sfr_name(adr)
        if ret == '':
            for k,v in list(vars(sfr1110).iteritems()):
                if v is adr:
                    return k
        return ret



class sfr1112 (sfr111x):

    DPDNCTL = 0xa1
    REGTRM0 = 0xa2
    REGTRM1 = 0xa3
    REGTRM2 = 0xa4
    REGTRM3 = 0xa5
    REGTRM4 = 0xa6
    AOPT    = 0xa7

    PWR_I   = 0xac
    PROVAL  = 0xad
    PROSTA  = 0xae
    PROCTL  = 0xaf

    GPIO5   = 0xd1

    GPIO34  = 0xd3

    DACLSB  = 0xf4

    dict_id = {0x2a:'CAN1112A', \
               0x2c:'CAN1112B'}

    def __init__ (me, revid=0):
        sfr11xx.__init__ (me, revid)
        me.inc = 0 # CAN1112 power-on I2CSLV in non-INC mode
        me.nbyte = 2
        me.dummy = 2
        me.nvmsz = 0x4200
        me.nvmmsk = 0x7fff # address width
        me.trimsz = 5
        me.trimnum = 6

    def get_sfr_name (me, adr):
        ret = super(sfr1112,me).get_sfr_name(adr)
        if ret == '':
            for k,v in list(vars(sfr1112).iteritems()):
                if v is adr:
                    return k
        return ret




TRUE  = 1 # ACK, YES
FALSE = 0 # NAK, NO

class i2c:
    '''
    i2c class hierarchy
    -------------------
         i2c
             \
               aardvark_i2c
             / (aardv.py)
    aardvark
    
    '''
    def enum (me): raise NotImplementedError()
    def baud (me, ask): raise NotImplementedError()
    def i2cw (me, wdat): raise NotImplementedError()
    def read (me, dev, adr, rcnt, rpt=FALSE): raise NotImplementedError()

    def write (me, dev, adr, wdat): # SMB write
        return me.i2cw ([dev,adr]+wdat)

    def probe (me):
        print 'Searching I2C slave.....'
        hit = []
        for dev in range(0x80):
            if me.i2cw ([dev]):
                print 'device 0x%02x found' % (dev)
                hit += [dev]
        return hit



def choose_i2cmst (rpt=TRUE):
    from cynpy.aardv import aardvark_i2c
    i2cmst = 0
    if aardvark_i2c().enum (rpt) > 0:
        i2cmst = aardvark_i2c(0)

    return i2cmst



if __name__ == '__main__':

    i2cmst = choose_i2cmst ()

    from cynpy.basic import *
    if not no_argument ():
        if i2cmst!=0:
            if   sys.argv[1]=='probe' : print i2cmst.probe ()
            elif sys.argv[1]=='baud'  : print i2cmst.baud (argv_dec[2])
            elif sys.argv[1]=='write' : print i2cmst.i2cw (argv_hex[2:])
            elif sys.argv[1]=='read'  : print '0x%02X' % i2cmst.read (argv_hex[2], argv_hex[3], 2, TRUE)[0]
            else: print "command not recognized"




from cynpy.can11xx import cani2c, sfr, TRUE, FALSE

class updprl (cani2c):
    '''
    base class of ams and csp
    '''
    OrdrsType = ['SOP','SOP\'','SOP"','SOP\'_Debug','SOP"_Debug','Hard Reset','Cable Reset']
    CtrlMsg = ['Rsvd0','GoodCRC','GotoMin','Accept','Reject','Ping','PS_RDY','Get_Source_Cap', \
               'Get_Sink_Cap','DR_Swap','PR_Swap','VCONN_Swap','Wait','Soft_Reset','Rsvd14','Rsvd15']
    DataMsg = ['Rsvd0','Source_Capabilities','Request','BIST','Sink_Capabilities','Battery_Status','Alert','Get_Country_Info', \
               'Rsvd8','Rsvd9','Rsvd10','Rsvd11','Rsvd12','Rsvd13','Rsvd14','Vendor_Defined']


    def __init__ (me, i2cmst, deva, ordrs=0):
        '''
        don't enable auto-TX-GoodCRC initially
            1. for Mode0 doesn't need
            2. keep silence when DUT speak
        enable all ordrs RX for SOP* query
        '''
        me.MyMsgId = 0
        me.DataRole = 0 # '1' in SOP'/SOP"
        me.PortRole = 0 # 0/1: SOURCE/CABLE
        me.SpecRev = 1 # PD_R2
        me.TxOrdrs = ordrs & 0x07

        me.TxBuffer = []

        cani2c.__init__ (me, i2cmst, deva)

        me.prltx = sfr(me, me.sfr.PRLTX, -1, TRUE)

        me.sfrwx (me.sfr.RXCTL,[0x7f])


    def compose_msg (me, msgtype, DO=[]):
        '''
        construct message and send
        DO is 32-bit
        return FALSE if discarded
        '''
        NDO = len(DO)
        me.TxBuffer = \
                [(   msgtype &0x1f) | ((me.DataRole&0x01) <<5) | ((me.SpecRev&0x03) <<6), \
                 (me.PortRole&0x01) | ((me.MyMsgId &0x07) <<1) | ((   NDO    &0x07) <<4)]
        for xx in range(NDO):
            me.TxBuffer += [(DO[xx])    &0xff, \
                            (DO[xx]>>8) &0xff, \
                            (DO[xx]>>16)&0xff, \
                            (DO[xx]>>24)&0xff ]

        me.sfrwx (me.sfr.TXCTL,[0x38 | me.TxOrdrs]) # turn on prea/CRC32/EOP
        me.sfrwx (me.sfr.STA1, [0xff]) # clear STA1
        me.sfrwx (me.sfr.FFSTA,[0x00]) # clear FIFO
        me.sfrwx (me.sfr.FFCTL,[0x40]); me.sfrwx (me.sfr.FFIO, me.TxBuffer[:-1]) # first
        me.sfrwx (me.sfr.FFCTL,[0x80]); me.sfrwx (me.sfr.FFIO, me.TxBuffer[-1:]) # last

        sta1 = me.sfrrx (me.sfr.STA1,1)[0]
        if (sta1&0x30)==0x10: # check not discarded
            return TRUE # success
        else:
            return FALSE # discarded


    def msg_tx (me, msgtype, DO=[]):
        '''
        send message and check GoodCRC
        DO is 32-bit
        '''
        me.prltx.msk (0xff, 0x08) # enable auto-RX-GoodCRC
        me.sfrwx (me.sfr.STA0, [0xff]) # clear STA0
        ret = ''
        if me.compose_msg (msgtype, DO): # success
            sta0 = 0
            for xx in range(20): # wait for something rcvd in a period of time
                if sta0 & 0xfc: break;
                sta0 = me.sfrrx (me.sfr.STA0,1)[0]
            if sta0 == 0x00: ret = 'no GoodCRC response, '+ me.OrdrsType[me.TxOrdrs-1]
            elif (sta0 & 0x40): # GoodCRC rcvd
                me.MyMsgId += 1
                me.MyMsgId &= 7
                # success
            else: # sta0 != 0x0a
                ret = 'bad response, STA0:%02x' % (sta0)
        else:
            ret = 'message discarded'

        me.prltx.pop ()
        return ret




from cynpy.updprl import *

class csp (updprl):
    '''
    CC-ISP for can11xx
    '''

    def csp_prefix (me, txlst):
        '''
        send a UVDM for entering Mode 0
        and send the 'txlst' as a CSP command
        CRC32 will be turnned off
        '''
        err = me.msg_tx (15,[0x412a412a])
        if err == '':
#           msvcrt.getch()
            me.sfrwx (me.sfr.TXCTL,[0x28 | me.TxOrdrs]) # turn off CRC32
            me.sfrwx (me.sfr.FFSTA,[0x00]) # clear FIFO
            me.sfrwx (me.sfr.STA0, [0xff]) # clear STA0
            me.sfrwx (me.sfr.STA1, [0xff]) # clear STA1
            me.sfrwx (me.sfr.FFCTL,[0x40]); me.sfrwx (me.sfr.FFIO, txlst[:-1]) # first
            me.sfrwx (me.sfr.FFCTL,[0x80]); me.sfrwx (me.sfr.FFIO, txlst[-1:]) # last
            sta1 = me.sfrrx (me.sfr.STA1,1)[0]
            sta1 = me.sfrrx (me.sfr.STA1,1)[0]
            sta1 = me.sfrrx (me.sfr.STA1,1)[0] # a little delay make the extra data disapeared??
            
            if (sta1&0x30)==0x10: return ''
            else:                 return 'CSP cmd discardrd, 0x%02x' % (sta1)
        else:
            return err

        
    def probe (me, rpt=0):
        '''
        query each SOP* for searching responding target(s)
        '''
        me.prltx.msk (0xff, 0x08) # enable auto-RX-GoodCRC
        sav = me.TxOrdrs
        fnd = []
        for xx in range(1,6):
            me.TxOrdrs = xx
            ret = me.csp_prefix ([0x00,0x01]) # no-data CSP write
            if rpt:
                print me.OrdrsType[xx-1],
                if ret == '':
                    print 'responded'
                else:
                    print '...'
            if ret == '':
                fnd += [xx]

        me.prltx.pop ()
        me.TxOrdrs = sav
        return fnd


    def cspw (me, adr, io, wdat):
        assert len(wdat) > 0 and \
               len(wdat)+2 <= me.sfr.bufsz, 'invalid data size, %d' % (len(wdat))
        ret = me.csp_prefix ([adr, 0x01|((io&0x01)<<1)] + wdat)

        if ret != '':
            print ret
            return 0 # 0-byte written
        else:
            return len(wdat)


    def cspr (me, adr, io, cnt):
        assert cnt <= me.sfr.bufsz and cnt > 0, 'invalid read data count, %d' % (cnt)
        ret = []
        err = me.csp_prefix ([adr, 0x00|((io&0x01)<<1), cnt-1])
        if err == '': # continue to receive the CSP read data
            sta0 = 0
            for xx in range(20): # wait for something rcvd
                if sta0 & 0xfc: break;
                sta0 = me.sfrrx (me.sfr.STA0,1)[0] & 0xfe
            if sta0 == 0x12 or sta0 == 0x10: # why 0x10 ??
                staf = me.sfrrx (me.sfr.FFSTA,1)[0]
                if staf == 0x80:
                    print 'no CSP read data returned'
                else:
                    me.sfrwx (me.sfr.FFCTL,[0x40]) # first
                    ret = me.sfrrx (me.sfr.FFIO, cnt)
            elif sta0 == 0x00:
                print 'CSP read not responded'
            else:
                print 'CSP responds a bad data, 0x%02x' % (sta0)
        else: # print the error message during the UVDM
            print err

        return ret


        

if __name__ == '__main__':
    '''
    % python csp.py [cmd|SOP*] [cmd] [...]
    % python csp.py query
    % python csp.py 1 read bb
    '''
    import cynpy.i2c as i2c
    i2cmst = i2c.choose_i2cmst (rpt=i2c.FALSE)

    import sys
    import cynpy.basic as cmd
    if not cmd.no_argument ():
        if i2cmst != 0:
            if sys.argv[1]=='query' : # query SOP*
                cspbdg = csp(i2cmst, 0x70) # no SOP*
                print cspbdg.probe (1)
            else:
                if cmd.argv_hex[1]>0 and cmd.argv_hex[1]<=5: # assign SOP* by command line
                    cspbdg = csp(i2cmst, 0x70, cmd.argv_hex[1])
                    sys.argv = sys.argv[1:]
                    cmd.argv_hex = cmd.argv_hex[1:]
                    cmd.argv_dec = cmd.argv_dec[1:]
                else:
                    cspbdg = csp(i2cmst, 0x70, 5)

                cspbdg.prltx.msk (0xff, 0x08) # enable auto-RX-GoodCRC
                from cynpy.can11xx import tstcsp
                tstmst = tstcsp(cspbdg)
                cmd.tstmst_func (tstmst)
                cspbdg.prltx.pop ()




import random
import time

from cynpy.basic import check_break

class atm (object):
    '''
    purely a method-only object
    for those SFR master, sfri2c and sfrcsp
    '''

    def sfr_form (me, adr, cnt=16):
        print me.sfrri, adr
        print 'sfr_dump: 0x%02x 0x%02x' % (adr,cnt)
        if ((adr&0x0f)+cnt<=16 and cnt<=8): # in one line
            print '0x%02x:' % adr,
            r_dat = me.sfrri (adr,cnt)
            assert len(r_dat)==cnt, 'sfr read failed'
            for i in range(cnt): print '%02x' % r_dat[i],
        else:
            pos = adr&0x0f
            for ali in range(adr&0xf0,(adr+cnt+15)&0x1f0,0x10):
                print '0x%02x:' % ali,
                r_dat = me.sfrri (ali,16-pos)
                assert len(r_dat)==(16-pos), 'sfr read failed'
                for i in range(0x10):
                    if (i&0x07==0 and i>0): print ' ',
                    if (ali+i<adr or ali+i>=adr+cnt): print '..',
                    else: print '%02x' % r_dat[i-pos],
                print
                pos = 0


    def nvmset (me, ofs):
        assert me.sfrrx (me.sfr.MISC,1)[0] & 0x08, 'MCU is runnung'
        msk = me.sfr.nvmmsk
        me.sfrwx (me.sfr.OFS, [(ofs&msk)&0xff]) # OTP offset [7:0]
        me.sfrwx (me.sfr.DEC, [((ofs&msk)|(0xa000&~msk))>>8]) # OTP offset [??:8], ACK for OTP access


    def nvmrx (me, cnt): # NINC mode
        ret = me.sfrrx (me.sfr.NVMIO, cnt)
        assert len(ret)==cnt, 'NVM read failed'
        return ret


    def nvm_form (me, ofs, cnt):
        assert ofs>=0 and cnt>0 and (ofs+cnt)<=me.sfr.nvmsz, 'out of range'
        print 'remember to halt MCU in advance'
        me.nvmset (ofs)
        if ((ofs&0x0f)+cnt<=16 and cnt<=8): # in one line
            print '0x%04x:' % ofs,
            r_dat = me.nvmrx (cnt)
            for i in range(cnt): print '%02x' % r_dat[i],
            print
        else:
            print 'nvm_form: 0x%04x, %0d' %(ofs,cnt)
            s_pos = ofs&0x0f
            lines = range(ofs&0xfff0,(ofs+cnt+15)&0xfff0,0x10)
            e_pos = 0x0f & (cnt - (16-s_pos))
            for ali in lines:
                print '0x%04x:' % (ali&0x7fff),
                if ali==lines[-1] and e_pos: num = e_pos
                else: num = 16-s_pos
                r_dat = me.nvmrx (num)
                for i in range(0x10):
                    if (i&0x07==0 and i>0): print ' ',
                    if (ali+i<ofs or ali+i>=ofs+cnt): print '..',
                    else: print '%02x' % r_dat[i-s_pos],
                endstr = '  '
                for i in range(0x10):
                    if (ali+i<ofs or ali+i>=ofs+cnt or
                        r_dat[i-s_pos]<ord(' ') or r_dat[i-s_pos]>ord('~')): endstr += '.'
                    else: endstr += chr(r_dat[i-s_pos])
                print endstr
                s_pos = 0
        me.sfrwx (me.sfr.DEC, [(ofs+cnt)>>8])


    def loopr (me, period, plist): # looped read and print
        if (len(plist)>0):
            print me.sfrrx
            print 'looped read, press any key.....'
            cnt = 0
            while 1:
                print "\r%0d:" % cnt,
                for xx in range (len(plist)):
                    try:
                        r_dat = me.sfrrx (int(plist[xx],16),1)[0]
                        print " %02x: %02x" %(int(plist[xx],16),r_dat),
                        cnt += 1
                    except:
                        print " %02x: --" %(int(plist[xx],16)),
                if check_break (): break


    def loopw (me, period, plist): # looped write/read test
        if (len(plist[0])>0):
            print me.sfrwx
            print 'looped write/read test, press any key.....'
            cnt = 0
            while 1:
                print "\r%0d:" % cnt,
                for xx in range (len(plist)):
                    wdat = random.randint(0,255)
                    me.sfrwx (int(plist[xx],16), [wdat]);
                    print " %02x: %02x" % (int(plist[xx],16),wdat),
                    r_dat = me.sfrrx (int(plist[xx],16),1)[0]
                    if r_dat!=wdat:
                        print " failed: %02x returned" % (r_dat)
                        exit (-1)
                if check_break (): break
                cnt += 1


    def preset_adc (me):
        me.sfrwx (me.sfr.DACLSB,[0x06]) # enable DAC1/COMP
        me.sfrwx (me.sfr.DACCTL,[0x00])
        me.sfrwx (me.sfr.SAREN, [0xff])

    def get_adc8 (me, chn): # multiple channel
        me.preset_adc ()
        me.sfrwx (me.sfr.DACEN, [chn])
        me.sfrwx (me.sfr.DACCTL,[0x0d]) # 8-bit once
        ret = []
        msk = 0x01
        for xx in range(8):
            if chn & (msk<<xx):
                ret += [8 * me.sfrrx (me.sfr.DACV0 + xx, 1)[0]] # mV
        return ret

    def get_adc10 (me, chn): # single channel
        me.preset_adc ()
        me.sfrwx (me.sfr.DACEN, [0x01 << chn])
        me.sfrwx (me.sfr.DACCTL,[0x4d]) # 10-bit once (not stable)
##        print \
##            2 * (4 * me.sfrrx (me.sfr.DACV0 + chn, 1)[0] \
##                  + (me.sfrrx (me.sfr.DACLSB, 1)[0] & 0x03)),
        me.sfrwx (me.sfr.DACCTL,[0x4f]) # 10-bit loop
        me.sfrwx (me.sfr.DACCTL,[0x00])
        return \
            2 * (4 * me.sfrrx (me.sfr.DACV0 + chn, 1)[0] \
                  + (me.sfrrx (me.sfr.DACLSB, 1)[0] & 0x03))


    def pre_prog (me, hiv=0): # provide high voltage
        assert hiv==0 or hiv==1, 'error argument, hiv'
        if me.sfr.name=='CAN1108' or \
           me.sfr.name=='CAN1111':
            rlst = []
            print 'Please provide VPP (6.5V) on VC1'
        else:
            rlst = \
                me.sfrrx (me.sfr.PWR_V,1) + \
                me.sfrrx (me.sfr.SRCCTL,1) # save PWR_V
            if hiv > 0: # hiv=0 to emulate
                me.sfrwx (me.sfr.PWR_V, [125]) # set VIN=10V

            print 'pre-VIN:',
            for xx in range(3):
                print '%5.2f' % (10.0 * me.get_adc10 (0) / 1000),
            print 'mV'

            me.sfrwx (me.sfr.SRCCTL, [rlst[1] | 0x40]) # set HVLDO high voltage

            if me.sfr.name.find ('CAN1110')==0:
                me.sfrwx (me.sfr.NVMCTL, [0x10,0x12,0x32]) # set VPP,TM,PROG

        return rlst


    def pst_prog (me, rlst): # resume 5V
        if not \
          (me.sfr.name=='CAN1108' or \
           me.sfr.name=='CAN1111'):
            if me.sfr.name.find ('CAN1110')==0:
                me.sfrwx (me.sfr.NVMCTL, [0x12,0x10,0x00]) # clr PROG,TM,VPP
            me.sfrwx (me.sfr.SRCCTL, [rlst[1] &~0x40]) # recover V5 (HVLDO)

            print 'pos-VIN:',
            print '%5.2f' % (10.0 * me.get_adc10 (0) / 1000),
            me.sfrwx (me.sfr.PWR_V, [rlst[0]]) # recover VIN
            for xx in range(3):
                print '%5.2f' % (10.0 * me.get_adc10 (0) / 1000),
            print 'mV'


    def nvm_prog (me, adr, wlst, hiv=0): # hiv=0 to emulate
        """
        program @adr those in list 'wlst' byte-by-byte
        slowly write for PROG timing
        """
        me.nvmset (adr)
        rlst = me.pre_prog (hiv)
        for xx in range(len(wlst)):
            me.sfrwx (me.sfr.NVMIO, [wlst[xx]])
        me.pst_prog (rlst)
        sav = me.sfrrx (me.sfr.DEC, 1)[0]
        me.sfrwx (me.sfr.DEC, [(me.sfr.nvmmsk >> 8) & sav]) # clear ACK


    def test (me):
        pass


    def get_file (me, memfile):
        """
        load file and return the array with byte-by-byte format
        """
        print memfile
        f = 0
        lines = []
        if memfile[-4:].lower() == '.bin':
            f = open (memfile,'rb')
            lines = map(ord,list(f.read()))
        elif memfile[-7:].lower() == '.1.memh' or \
             memfile[-7:].lower() == '.2.memh':
            f = open (memfile,'r')
            for xx in f.readlines ():
                text = xx.split()[0] # only the 1st word
                if text != '': # ignore empty
                    assert len(text) == 2 * me.sfr.nbyte, \
                           'invalid format for %s' % (me.sfr.name)
                    if me.sfr.nbyte == 2:
                        lines.append (int(text[2:4],16))
                    lines.append (int(text[0:2],16))
        else:
            print 'ERROR: file format'

        print '%d (0x%04x) byte(s)' % (len(lines),len(lines))

        rem = len(lines) % me.sfr.nbyte
        if rem > 0:
            print 'append 0xFF for programming unit,', me.sfr.nbyte - rem
            for xx in range(me.sfr.nbyte - rem):
                lines.append (0xff)

        return lines


    def nvm_prog_block (me, addr, wrcod, rawsz, hiv=0, block=256):
        """
        program the byte-by-byte array 'wrcod' in to the NVM block-by-block
        SFR-by-CSP: limit block size by CSP buffer and dummy
        SFR-by-I2C: 100KHz write for PROG timing
        """
        assert block > 0, 'block size must be positive'
        me.nvmset (addr)
        rlst = me.pre_prog (hiv)
        start = time.time ()

        for xx in range(0, len(wrcod), block):
            wcnt = block if xx+block <= len(wrcod) else len(wrcod)-xx
            me.sfrwx (me.sfr.NVMIO, wrcod[xx:xx+wcnt])

        print "%.1f sec" % (time.time () - start)
        me.pst_prog (rlst)
        ofs = me.sfrrx (me.sfr.OFS, 1)[0]
        dec = me.sfrrx (me.sfr.DEC, 1)[0]
        endadr = (ofs+dec*256) & me.sfr.nvmmsk
        me.sfrwx (me.sfr.DEC, [endadr>>8]) # clear ACK

        print ('ERROR: 0x%04x' % (endadr)) if endadr != addr + rawsz else 'complete'


    def nvm_upload_block (me, memfile, hiv=0):
        """
        load the memory file 'memfile' for uploading to NVM
        """
        wrcod = me.get_file (memfile)
        me.nvm_prog_block (0, wrcod, len(wrcod), hiv)


    def show_mismatch (me, adr, dat, exp, num, limit=32):
        """
        for counting number of mismatch
        and show mismatch messages
        """
        if num  < limit: print '0x%04X : %02X (!=%02X)' % (adr, dat, exp)
        if num == limit: print 'further mismatch(es) will be suppressed',
        return 1


    def nvm_chk_blank (me, start, end, mismatch, block=256): # the byte 'end' is not compared
        """
        check if those contents are all '1' block-by-block
        """
        print 'blank check from 0x%04x to 0x%04x' % (start, end),
        me.nvmset (start)
        while start < end:
            rcnt = block if start+block <= end else end-start
            rdat = me.nvmrx (rcnt)
            for yy in range(len(rdat)):
                if rdat[yy] != 0xff:
                    mismatch += me.show_mismatch (start+yy, rdat[yy], 0xff, mismatch)
            start += rcnt
        return mismatch


    def nvm_comp_block (me, args, block=256):
        print 'compare contents to',
        expcod = me.get_file (args[0]) # memfile
        if len(args) > 1: # exception(s)
            for it in args[1:]:
                [adr_h, text] = it.split('=')
                adr = int(adr_h,16)
                if text[0]=='\\':
                    expcod[adr] = int(text[1:],16)
                else:
                    for tt in range(len(text)):
                        if adr+tt < len(expcod):
                            expcod[adr+tt] = ord(text[tt])
        mismatch = 0
        me.nvmset (0)
        start = time.time ()
        for xx in range(0,len(expcod),block):
            rcnt = block if xx+block <= len(expcod) else len(expcod)-xx
            rdat = me.nvmrx (rcnt)
            for yy in range(len(rdat)):
                if rdat[yy] != expcod[yy+xx]:
                    mismatch += me.show_mismatch (yy+xx, rdat[yy], expcod[yy+xx], mismatch)

#       mismatch = me.nvm_chk_blank (xx + rcnt, me.sfr.nvmsz, mismatch, block)

        print '%.1f sec' % (time.time () - start)
        ofs = me.sfrrx (me.sfr.OFS, 1)[0]
        dec = me.sfrrx (me.sfr.DEC, 1)[0]
        endadr = (ofs+dec*256) & me.sfr.nvmmsk
        me.sfrwx (me.sfr.DEC, [endadr>>8]) # clear ACK

        print ('mismatch: %s' % (mismatch)) if mismatch else 'complete'


    def get_trim (me):
        '''
        search the trim MTTable
        get the newest entry
        '''
        me.nvmset (0x940)
        ret = []
        rdat = me.nvmrx (me.sfr.trimsz *me.sfr.trimnum)
        for xx in range(me.sfr.trimnum):
            cnt_not_ff = 0
            for yy in range(me.sfr.trimsz):
                if rdat[xx*me.sfr.trimsz + yy] != 0xff:
                    cnt_not_ff += 1
            if cnt_not_ff == 0 and xx > 0: # found
                ret = rdat[(xx-1)*me.sfr.trimsz : xx*me.sfr.trimsz]
                break

        ofs = me.sfrrx (me.sfr.OFS, 1)[0]
        dec = me.sfrrx (me.sfr.DEC, 1)[0]
        endadr = (ofs+dec*256) & me.sfr.nvmmsk
        me.sfrwx (me.sfr.DEC, [endadr>>8]) # clear ACK

        return ret





from cynpy.updprl import *

class ams (updprl):
    '''
    Atomic Message Sequence
    1. auto-respond
    2. initiator
    Note:
    a. enable auto-TX/RX-GoodCRC during TX/RX
    b. via AARDVARK (USB-I2C bridge) on virtual machine doesn't work
    '''

    def __init__ (me, i2cmst, deva, check_kb_func, ordrs=0):
        super(ams,me).__init__ (i2cmst, deva, ordrs)
        me.check_kb_func = check_kb_func

        me.pdo = []
        me.rdo = 0

        me.str = '' # command string
        me.kb = ord('1') # command from keyboard
        me.mode = 0 # 0/1/2 : RX/TX/PPS
        me.ppstime = 0
        me.SpecRev = 2 # PD_R3


    def ams_quick_tx_than_wait (me, cmd, do=[]):
        me.sfrwx (me.sfr.STA0, [0xff]) # clear STA0, prepare for the 1st 'quick' upd_rx
        me.sfrwx (me.sfr.STA1, [0xff]) # clear STA1
        me.msg_tx (cmd, do)

        msg = 'TX - %02X%02X' % (me.TxBuffer[1],me.TxBuffer[0]) \
                                          + (':' if len(do) > 0 else '')
        for yy in range(len(do)): # NDO
            msg += ' %08X' % do[yy]
        print '%s (%s)' % (msg, me.DataMsg[cmd] if len(do) > 0 else me.CtrlMsg[cmd])

        return me.upd_rx (quick=1)


    def pps_request (me, cmd):
        '''
        with a command or a target voltage
        don't check voltage range for stressing DUT
        '''
        pos_minus = (me.rdo >> 28) & 0x07 - 1
        ppscur = (me.rdo >> 9) & 0x7ff
        me.rdo &= ~0x000ffe00
#       ppsmax = ((me.pdo[pos_minus] >> 17) & 0x7f) * 5 # 20mV
#       ppsmin = ((me.pdo[pos_minus] >> 8)  & 0x7f) * 5 # 20mV

        if len(cmd) == 1:
            if   cmd[0] == '+':
                me.rdo |= (ppscur + 1) << 9
            elif cmd[0] == '-':
                me.rdo |= (ppscur - 1) << 9
            elif cmd[0] == '*':
                me.rdo |= ppscur << 9
        if len(cmd) > 1:
            ppsv = cmd.split('=')[-1]
            if ppsv.isdigit():
                me.rdo |= (int(ppsv) / 20) << 9

        if me.rdo & 0x000ffe00: # target voltage is valid
            (ndo,mtyp,do) = me.ams_quick_tx_than_wait (2, [me.rdo])
            if me.ppstime > 0:
                me.ppstime = 0
            if (ndo,mtyp) == (0,3): # Accept
                me.upd_rx () # PS_RDY


    def ams_request (me):
        '''
        '''
        pos_minus = (me.kb - ord('1')) & 0x07
        me.rdo = (pos_minus+1) << 28
        ret = (FALSE,FALSE) # not a APDO, not accepted

        if len(me.pdo) > pos_minus:
            if (me.pdo[pos_minus] >> 30) == 3: # APDO, max voltage
                ret = (TRUE,ret[1])
                ppsmax = ((me.pdo[pos_minus] >> 17) & 0x7f) * 5 # 20mV
                me.rdo |= ((ppsmax & 0x7ff) << 9) | (me.pdo[pos_minus] & 0x7f)
            else:
                me.rdo |= me.pdo[pos_minus] & 0x3ff | ((me.pdo[pos_minus] & 0x3ff) << 10)

        (ndo,mtyp,do) = me.ams_quick_tx_than_wait (2, [me.rdo])
        if (ndo,mtyp) == (0,3): # Accept
            ret = (ret[0],TRUE)
            me.upd_rx () # PS_RDY

        return ret


    def ams_get_source_cap (me):
        '''
        '''
        (ndo,mtyp,do) = me.ams_quick_tx_than_wait (7)
        if ndo > 0 and mtyp == 1: # SourceCap
            me.pdo = do
            if me.ams_request () == (TRUE,TRUE): # a APDO accepted
                me.mode = 2
                print '[AMS_PPS]'


    def ams_tx (me):
        '''
        '''
        key = me.check_kb_func ()
        if key == ord('q'): # 'q'
            me.mode = 0
            print '[AMS_RX]'

        elif key == ord('g'): # Get_Source_Cap
            me.ams_get_source_cap ()

        elif key >= ord('0') and key <= ord('7'): # Request
            me.kb = key
            if me.ams_request () == (TRUE,TRUE): # a APDO accepted
                me.mode = 2
                print '[AMS_PPS]'

        elif key > 0:
            me.kb = key


    def ams_pps (me, sleep_func):
        '''
        '''
        if me.ppstime >= 0:
            me.ppstime += 1
        sleep_func (0.01)
        if me.ppstime == 666: # about 10 sec
            me.pps_request ('*') # reset me.ppstime in this

        key = me.check_kb_func ()
        if me.str == '':
            if key == ord('q'): # 'q'
                me.mode = 0
                print '[AMS_RX]'
            elif key == ord('+') or \
                 key == ord('-') or \
                 key == ord('*'): # inc/dec/same
                me.pps_request (chr(key))
            elif key == ord('/'): # switch SinkPPSPeriodTimer
                me.ppstime = 0 if me.ppstime < 0 else -1
                print 'switch SinkPPSPeriodTimer', 'ON' if me.ppstime == 0 else 'OFF'
            elif key == ord('='): # set new voltage
                me.str = '='
                print me.str, '\x0D',

            elif key >= ord('0') and key <= ord('7'): # Request
                me.kb = key
                if me.ams_request () == (FALSE,TRUE): # a PDO accepted
                    me.mode = 1
                    print '[AMS_TX]'

            elif key > 0:
                me.kb = key

        else:
            if key == 13: # press 'enter' to exec 'me.str'
                print me.str
                me.pps_request (me.str)
                me.str = '' # reset 'me.str'
            elif key > ord(' '):
                me.str += chr(key)
                print me.str, '\x0D',


    def ams_rx (me):
        '''
        get/decode RX a message, respond if needed
        '''
        (ndo,mtyp,do) = me.upd_rx ()
        if ndo > 0: # data message
            if mtyp == 1: # SourceCap
                me.pdo = do
                if me.ams_request () == (TRUE,TRUE): # a APDO accepted
                    me.mode = 2
                    print '[AMS_PPS]'
                
        elif ndo == 0: # control message
            if mtyp < 0: # Hard/Cable Reset
                print 'RX -', me.OrdrsType[mtyp+7]

        else: # 'q' pressed
            me.mode = 1
            print '[AMS_TX]'


    def upd_rx (me, quick=0):
        '''
        wait for an auto-TX-GoodCRC sent, then,
        get STA0, get FFSTA, get RX data all take time
        if another RX arrive during these, those data is corrupted
        quick=1 to not clear STA* for a new message may have arrived
            (TX caller is responsible for clearing the STA*)
        '''
        me.prltx.msk (0xff, 0x80) # enable auto-TX-GoodCRC
        if quick == 0:
            me.sfrwx (me.sfr.STA0, [0xff]) # clear STA0
            me.sfrwx (me.sfr.STA1, [0xff]) # clear STA1
        (sta0,sta1) = (0,0)
        (ndo,mtyp,do,key) = (0,0,[],0)
        while not (sta1 & 0x40) and \
              not (sta0 & 0x80):
            sta0 = me.sfrrx (me.sfr.STA0,1)[0]
            sta1 = me.sfrrx (me.sfr.STA1,1)[0]
            key = me.check_kb_func ()
            if key > 0: # key pressed
                if key >= ord('0') and key <= ord('7'):
                    me.kb = key
                    print chr(key) # this 'kb' will effect the next Nego
                elif key == ord('q'):
                    ndo = -1
                    break
                else:
                    me.kb = key # save the 'kb' for main loop usage

        if not ndo < 0: # not break

            if sta1 & 0x40: # auto-TX-GoodCRC sent
                ffcnt = me.sfrrx (me.sfr.FFSTA,1)[0] & 0x3f
                assert ffcnt > 0, 'empty FIFO received'

                me.sfrwx (me.sfr.FFCTL, [0x40]) # first
                rdat = me.sfrrx (me.sfr.FFIO, ffcnt)
                assert ffcnt == len(rdat), 'FIFO read error'

                ndo = (rdat[1] >> 4) & 0x07
                mtyp = rdat[0] & 0x1f
                do = [0] * ndo
                msg = 'RX - %02X%02X' % (rdat[1],rdat[0]) + (':' if ndo > 0 else '')
                for yy in range(ndo):
                    for xx in range(4):
                        do[yy] += rdat[yy*4+2+xx]*(256**xx)
                    msg += ' %08X' % do[yy]
                print msg

            else: # Hard/Cable Reset rcvd
                prls = me.sfrrx (me.sfr.PRLS,1)[0] & 0x70
                if   prls == 0x60: mtyp = -2 # Hard Reset
                elif prls == 0x70: mtyp = -1 # Cable Reset
                else:
                    raise NameError('not recognized ordered set')

        me.prltx.pop () # recover PRLTX settings
        return (ndo,mtyp,do)



if __name__ == '__main__':
    '''
    % python ams.py
    '''
    import cynpy.i2c as i2c
    i2cmst = i2c.choose_i2cmst (rpt=i2c.FALSE)

    import time as time
    import cynpy.basic as cmd
    if i2cmst != 0:
        tstmst = ams(i2cmst, 0x70, cmd.check_break, 1)

        '''
        enable auto-TX/RX-GoodCRC
        recover auto-TX/RX-GoodCRC setting
        '''
        tstmst.prltx.msk (0xff, 0x88) # enable auto-TX/RX-GoodCRC
        print '[AMS_RX]'
        while tstmst.kb != 27:
            if   tstmst.mode == 0: tstmst.ams_rx ()
            elif tstmst.mode == 1: tstmst.ams_tx ()
            elif tstmst.mode == 2: tstmst.ams_pps (time.sleep)

            if tstmst.kb == ord(' '): # toogle PD2/3
                tstmst.kb = ord('1') # reset/clear 'kb'
                tstmst.SpecRev = 3 - tstmst.SpecRev
                print '[PD%d0]' % (tstmst.SpecRev + 1)

        tstmst.prltx.pop () # recover PRLTX settings




from cynpy.sfr import *

class can11xx (object):
    '''
    can11xx hierarchy
    -----------------                                     csp
                                                         /
                        sfr { sfrmst               updrpl - ams
                                            {i2c  /
            sfr1108                         cani2c - tsti2c
           /                               /        /
    sfr11xx - sfr111x - sfr1110  }  can11xx      atm 
                     \                     \        \
                      sfr1112                - - - - tstcsp
                                                       {csp
    '''
    def __init__ (me):
        me.sfr = sfr11xx() # initial
        if me.is_master_rdy():
            revid = me.get_revid ()
#           print 'master is ready', revid
            if revid > 0: # found
                if sfr1108().check (revid): me.sfr = sfr1108(revid)
                if sfr1110().check (revid): me.sfr = sfr1110(revid)
                if sfr1112().check (revid): me.sfr = sfr1112(revid)
#       else:
#           print 'master is not ready'


    def is_master_rdy (me): raise NotImplementedError()
    def sfrwx (me, adr, wdat): raise NotImplementedError() # NINC mode
    def sfrrx (me, adr, cnt): raise NotImplementedError() # NINC mode
    def sfrri (me, adr, cnt): raise NotImplementedError() # INC mode


    def get_revid (me):
        sav = me.sfrrx (me.sfr.DEC, 1) # try slave
        if len(sav): # data returned
            me.sfrwx (me.sfr.DEC, [me.sfr.REVID])
            revid = \
            me.sfrrx (me.sfr.REVID, 1)[0] & 0x7f
            me.sfrwx (me.sfr.DEC, [sav[0]])
            return revid
        return 0



class cani2c (can11xx):
    def __init__ (me, busmst, deva, rpt=0):
        me.deva = deva
        me.busmst = busmst # SFR master (I2C)
        can11xx.__init__ (me) # SFR target

        if me.sfr.revid:
            if rpt:
                print 'I2C master finds %s, 0x%02x' % (me.sfr.name, me.deva)
            if me.sfr.inc == 1: # CAN1108/11
                me.sfrwx (me.sfr.I2CCTL, [me.sfrrx (me.sfr.I2CCTL,1)[0] | 0x01]) # we'll work in NINC mode


    def is_master_rdy (me):
        ''' Is this master ready for issuing things?
        '''
        return TRUE if me.busmst else FALSE


    def sfrwx (me, adr, wdat):
        return me.busmst.write (me.deva, adr, wdat)


    def sfrrx (me, adr, cnt):
        return me.busmst.read (me.deva, adr, cnt, FALSE)


    def sfrri (me, adr, cnt):
        sav = me.sfrrx (me.sfr.I2CCTL, 1)[0]
        setinc = sav & 0xfe if me.sfr.inc else sav | 0x01
        me.sfrwx (me.sfr.I2CCTL, [setinc]) # INC mode
        rdat = me.busmst.read (me.deva, adr, cnt, TRUE)
        me.sfrwx (me.sfr.I2CCTL, [sav])
        return rdat



from cynpy.atm import *

class tsti2c (cani2c, atm):
    pass


class tstcsp (can11xx, atm):
    def __init__ (me, busmst, rpt=0):
        me.busmst = busmst # SFR master (CSP)
        super(me.__class__, me).__init__ () # SFR target

        if me.sfr.revid and rpt:
            print 'CSP master finds %s, %d' % (me.sfr.name, me.busmst.TxOrdrs)


    def is_master_rdy (me):
        ''' Is this master ready for issuing things?
        '''
        return me.busmst.busmst.handle


    def sfrwx (me, adr, wdat):
        return me.busmst.cspw (adr, 1, wdat)


    def sfrrx (me, adr, cnt):
        return me.busmst.cspr (adr, 1, cnt)


    def sfrri (me, adr, cnt):
        return me.busmst.cspr (adr, 0, cnt)


    def insert_dummy (me, rawcod, block):
        '''
        load the memory file 'memfile' and insert dummys
        '''
        lowcod = [] # low-byte
        wrcod = []
        for xx in rawcod:                
            if len(lowcod)>0 or me.sfr.nbyte==1:
                if len(wrcod)%block > 0:
                    for yy in range(me.sfr.dummy):
                        wrcod += [0xdd]
                wrcod += lowcod + [xx]
                lowcod = []
            else:
                lowcod = [xx]

        return (len(rawcod), wrcod)


    def nvm_prog_block (me, addr, wrcod, rawsz, hiv=0):
        """
        override atm's method
        calc. block length
        insert dummy byte(s)
        """
        w_unit = (me.sfr.bufsz - me.sfr.nbyte - 2) \
                              / (me.sfr.nbyte + me.sfr.dummy) # CSP command needs 2 bytes
        block = (me.sfr.nbyte + me.sfr.dummy) * w_unit \
               + me.sfr.nbyte # optimize block size by CSP buffer

        (rawsz, dmycod) = me.insert_dummy (wrcod, block)
        super(me.__class__, me).nvm_prog_block (addr, dmycod, rawsz, hiv, block)


    def nvm_comp_block (me, args, block=32):
        '''
        limit block size by CSP buffer
        '''
        super(me.__class__, me).nvm_comp_block (args, block)



class sfr (object):
    """
    to monitor I2C/CSP register
    to save read/modify time on communication channel
    """

    def __init__ (me, sfrmst, port, val=-1, dbmsg=FALSE):
        me.p = port # port/address
        me.v = [0] # one element for recovering
        me.sfrmst = sfrmst
        me.d = dbmsg
        if val<0: me.v[0] = me.sfrmst.sfrrx (port, 1)[0] # initial condition
        else:     me.v[0] = val                          # power-on value

    def __del__ (me):
        '''
        DIFFICULT TO CONTROL THIS
        SO THIS RECOVERY FUNCTION DOESN'T WORK YET
        '''
#       me.set (me, me.v[0])
        print 'SFR.%02X died' % (me.p)

    def doit (me):
        me.sfrmst.sfrwx (me.p, [me.v[-1]]) # main job of this class
        if me.d:
            print 'SFR.%02X: %02X, [' % (me.p,me.v[-1]), \
                  '%02X ' * len(me.v) % tuple(me.v)

    def get (me): return me.v[-1]

    def set (me, val, force=FALSE): # to set without push
        assert val==(val&0xff), '"val" out of range'
        chk = me.v[-1] != val
        me.v[-1] = val 
        if force or chk:
            me.doit ()

    def psh (me, val=-1, force=FALSE):
        if val<0:
            me.v += [ me.v[-1] ] # duplicate
        else:
            assert val==(val&0xff), '"val" out of range'
            me.v += [ val ] # push
            if force or me.v[-1] != me.v[-2]:
                me.doit ()

    def pop (me, force=FALSE): # resume
        tmp = me.v.pop ()
        if force or me.v[-1] != tmp:
            me.doit ()

    def msk (me, a, o=0):
        me.psh ((me.v[-1] & a) | o)



if __name__ == '__main__':
    '''
    % python can11xx.py [cmd] [argv2] [...]
    % python can11xx.py rev
    % python can11xx.py write bc 8
    '''
    import cynpy.i2c as i2c
    i2cmst = i2c.choose_i2cmst (rpt=i2c.FALSE)
    tstmst = tsti2c(busmst=i2cmst, deva=0x70)

    import cynpy.basic as cmd
    if not cmd.no_argument ():
        if i2cmst!=0:
            cmd.tstmst_func (tstmst)




from aardvark_py import *
from cynpy.i2c import *

class aardvark:
    def __init__ (me, p=-1):
        me.handle = 0
        if p >= 0:
            me.openChannel (p)

    def __del__ (me):
        if me.handle > 0:
            aa_close (me.handle)

    def openChannel (me, p):
        (number, ports, unique_ids) = aa_find_devices_ext(16, 16)
        if number > p:
            me.handle = aa_open (ports[p])
            if me.handle > 0:
                pass
            else:
                print 'Unable to open Aardvark device on port %d' % (p)
                print 'Error code = %d' % (me.handle)
        elif number > 0:
            print 'port', p, 'not found'
        else:
            print 'no AARDVARK port found'

    def enum (me, rpt=FALSE):
        (number, ports, unique_ids) = aa_find_devices_ext(16, 16)
        if rpt:
            print number, 'AARDVARK port(s) found'
            for xx in ports:
                print 'port %d' % (xx & ~AA_PORT_NOT_FREE),
                if xx & AA_PORT_NOT_FREE:
                    print 'is busy, skipped!!'
                else:
                    print 'v'*10
                    dev = aardvark(xx)
                    dev.aaShowVersion ()
        return number

    def aaShowVersion (me):
        assert me.handle > 0, 'no AARDVARK device opened'
        (sta, aaVer) = aa_version (me.handle)
        if (sta==AA_OK):
            print "struct AardvarkVersion {"
            print "\tsoftware: %x"      % aaVer.software
            print "\tfirmware: %x"      % aaVer.firmware
            print "\thardware: %x"      % aaVer.hardware
            print "\tsw_req_by_fw: %x"  % aaVer.sw_req_by_fw
            print "\tfw_req_by_sw: %x"  % aaVer.fw_req_by_sw
            print "\tapi_req_by_sw: %x" % aaVer.api_req_by_sw
            print "};"
        else:
            print "aa_version () failed!"

    def aaShowTargetPowerSta (me, sta):
        if   (sta==AA_TARGET_POWER_NONE):
            print "AA_TARGET_POWER_NONE %d" %sta
        elif (sta==AA_TARGET_POWER_BOTH):
            print "AA_TARGET_POWER_BOTH %d" %sta
        elif (sta==AA_INCOMPATIBLE_DEVICE):
            print "AA_INCOMPATIBLE_DEVICE %d" %sta
        else:
            print "aa_target_power failed! %d" %sta

    def aaSwitchTargetPower (me):
        assert me.handle > 0, 'no AARDVARK device opened'
        sta = aa_target_power (me.handle, AA_TARGET_POWER_QUERY)
    #   print aa_status_string (sta)
        if   (sta==AA_TARGET_POWER_BOTH):
            me.aaShowTargetPowerSta (aa_target_power (me.handle, AA_TARGET_POWER_NONE))
        elif (sta==AA_TARGET_POWER_NONE):
            me.aaShowTargetPowerSta (aa_target_power (me.handle, AA_TARGET_POWER_BOTH))
        else:
            print "TargetPowerSwitch failed! %d" %sta

    def aaSwitchPullup (me, ask): # '0' to ask, other for new setting
        assert me.handle > 0, 'no AARDVARK device opened'
    #   AA_I2C_PULLUP_QUERY
    #   AA_I2C_PULLUP_NONE
    #   AA_I2C_PULLUP_BOTH
        print aa_status_string (aa_i2c_pullup (me.handle, ask))
        # 0: none
        # others: OK


class aardvark_i2c (aardvark, i2c):
    def __init__ (me, p=-1):
        aardvark.__init__(me, p)
        if me.handle > 0:
            aa_configure (me.handle, AA_CONFIG_GPIO_I2C) # Ensure that the subsystem is enabled

    def baud (me, ask=0): # '0' to ask, other for new setting
        if not me.handle > 0: return 0 # no AARDVARK device opened'
        return aa_i2c_bitrate (me.handle, ask)

    def read (me, dev, adr, bycnt, i2cr1i, rpt=FALSE): # SMB read
        '''
        i2cr1i is not used in this I2C master
        '''
        if not me.handle > 0: return [] # no AARDVARK device opened'
        aa_i2c_write (me.handle, dev, AA_I2C_NO_STOP, array('B',[adr]))
        (r_cnt,r_dat) = aa_i2c_read (me.handle, dev, AA_I2C_NO_FLAGS, bycnt)
        if rpt:
            assert r_cnt == bycnt, 'I2C read failed, cnt:%d, exp:%d' \
                                                   % (r_cnt,bycnt)
            print '0x%02X: ' % adr,
            print (r_cnt,r_dat)
        return r_dat.tolist()

    def i2cw (me, wdat): # I2C write
        if not me.handle > 0: return 0 # no AARDVARK device opened'
        assert len(wdat) > 0, 'empty write data is not valid'
        if len(wdat) == 1:
            wdat += [0]
        return aa_i2c_write (me.handle, wdat[0], AA_I2C_NO_FLAGS, array('B',wdat[1:]))



if __name__ == '__main__':

    def test_only ():
        aa = aardvark_i2c(0)
        print 'query:', aa_configure (aa.handle, AA_CONFIG_QUERY)
        print aa_gpio_direction (aa.handle, 0x38)
        print aa_gpio_set (aa.handle, 0x38)
        print aa_gpio_get (aa.handle)
        print aa_gpio_set (aa.handle, 0x00)
        print aa_gpio_get (aa.handle)
        pass

    from cynpy.basic import *
    if not no_argument ():
        if   sys.argv[1]=='enum'  : aardvark().enum (rpt=TRUE)
        elif sys.argv[1]=='sw'    : aardvark(0).aaSwitchTargetPower ()
        elif sys.argv[1]=='pull'  : aardvark(0).aaSwitchPullup (int(sys.argv[2]))
        elif sys.argv[1]=='test'  : test_only ()
        else: print "command not recognized"




import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
      name='cynpy',
      version='0.15',
      description='Ray\'s public DVT utilities in Python 2.7',
      long_description=long_description,
      long_description_content_type="text/markdown",
      url='http://canyon-semi.com.tw',
      author='Ray Huang',
      author_email='rayjhuang@msn.com',
      license='MIT',
      packages=setuptools.find_packages(),
      package_data={
          '' : ['py_writer.bat']
          },
      zip_safe=False)
